%SGCOM StratiGraph comunication object.
%   SGCOM(STRUCT,STRATATYPE) returns a new StratiGraph communication
%   object. This class should not be called directly.
%
%   See also SGSET, SGGET, SGOPEN.
classdef sgcom < handle
  properties (Access = private)
    type, A, B, stratatype, mlstruct;
  end
  
  properties (Constant)
    MATRIX = 1;
    PAIR   = 2;
    PENCIL = 3;
    STATESPACE = 4;
    MATRIXPOLY = 5;
    FULLRANKMATRIXPOLY = 6;
  end
  
  methods
  
  function obj = sgcom(varargin) 
  %SGCOM StratiGraph comunication object.
  %   SGCOM(STRUCT,STRATATYPE) returns a new StratiGraph communication
  %   object. This function should not be called directly.
  %
  %   SGCOM('matrix',A,STRATATYPE)
  %   SGCOM('abpair',A,B,STRATATYPE)
  %   SGCOM('acpair',A,C,STRATATYPE)
  %   SGCOM('pencil',G,H,STRATATYPE) 
  %
  %   See also SGSET, SGGET, SGOPEN.

  % Initialize object
      obj.type = 0;
      obj.A    = [];
      obj.B    = [];
      obj.stratatype = 'orbit';
      obj.mlstruct   = [];

      % Return if no input arguments
      if ~nargin
        return
      end
      
      if isa(varargin{1},'sgcom')
        obj = varargin{1};
        return
      end
      
      narginchk(2,4);
      nargoutchk(0,1);
      
      if ischar(varargin{1})
        switch varargin{1}
          case 'matrix'
            if nargin == 4
              error('Incorrect number of inputs.');
            end
            obj.type       = obj.MATRIX;
            obj.A          = varargin{2};
            obj.B          = [];
            obj.stratatype = lower(varargin{3});
            obj.mlstruct   = mguptri(obj.A);
          case {'acpair', 'ac', 'obsv'}
            error('Not implemented for observability pairs.');
          case {'abpair', 'ab', 'ctrb'}
            obj.type       = obj.PAIR;
            obj.A          = varargin{2};
            obj.B          = varargin{3};
            obj.stratatype = lower(varargin{4});
            obj.mlstruct   = s2guptri('ab',obj.A,obj.B);
          case 'pencil'
            obj.type       = obj.PENCIL;
            obj.A          = varargin{2};
            obj.B          = varargin{3};
            obj.stratatype = lower(varargin{4});
            obj.mlstruct   = pguptri(obj.A,obj.B);
          otherwise
            error('Unknown type.');
        end
      elseif isobject(varargin{1})
        if nargin == 3
          error('Incorrect number of inputs.');
        end
        
        obj.mlstruct   = varargin{1};
        obj.stratatype = lower(varargin{2});
        obj.A          = [];
        obj.B          = [];
        
        if isa(obj.mlstruct,'mstruct')
          obj.type  = obj.MATRIX;
        elseif isa(obj.mlstruct, 'pstruct')
          obj.type  = obj.PENCIL;
        elseif isa(obj.mlstruct, 'mpstruct')
          obj.type  = obj.MATRIXPOLY;
        elseif isa(obj.mlstruct, 'ssstruct') && strcmp(obj.mlstruct.oftype,'ab')
          obj.type  = obj.PAIR;
        elseif isa(obj.mlstruct, 'ssstruct') && strcmp(obj.mlstruct.oftype,'ac')
          obj.type  = obj.PAIR;
        elseif isa(obj.mlstruct, 'ssstruct')
          obj.type  = obj.STATESPACE;
        else
          error(['The canonical structure %s does not exist in StratiGraph '...
                 'or the connection to StratiGraph is not implemented.'],... 
                class(obj.mlstruct))
        end
      else
        error('Illegal first argument.');
      end
			
      if (~isa(obj.stratatype,'java.lang.String') && ~ischar(obj.stratatype)) ...
            | ~strcmpi(obj.stratatype,{'orbit','bundle'}) %#ok<OR2>
        error('Unknown strata keyword.');
      end  
  
    end % End constructor
  end % End public methods
end % Classdef
