function constr = sggetconstraints(obj)
%SGGETCONSTRAINTS Return a list of available setup constraints.
%   Constr = SGGETCONSTRAINTS(Struct) returns for a canonical structure
%   object the list Constr with all available constraints for the
%   corresponding setup in StratiGraph, where Struct can be a canonical
%   structure object or the name of the canonical structure class as a
%   string. The returned list is a cell-array of strings.
%
%   See also SGSET.

narginchk(1,1);
nargoutchk(0,1);

if ~isa(obj,'mcsstruct') && ~ischar(obj)
  error('The first argument must be a string or a canonical structure object.');
end

if ischar(obj)
  sobj = meta.class.fromName(obj);
  pSetup = findobj(sobj.PropertyList,'Name','SGSetupName');
  setupName = pSetup.DefaultValue; % get SG setup name
elseif isa(obj,'mcsstruct')
  setupName = obj.SGSetupName;
end

if isempty(setupName)
  warning('MCS:sggetconstraints:InvalidSetup','Setup is not implemented in StratiGraph.');
  constr = [];
  return
end
  
try
  [sgc, winID, label] = sgdata('');
catch e
  if strcmp(e.identifier,'MCS:sgdata:NoInstance')
    error('MCS:sggetconstraints:NoInstance',['StratiGraph must be ' ...
                        'started to get available constraints.']);
  end
  rethrow(e)
end

% Check that MCS Toolbox Integrator is loaded into StratiGraph
import se.umu.stratigraph.core.PluginManager;
if ~PluginManager.isLoaded(winID,'MCS Toolbox Integrator')
  error('MCS:sggetconstraints:ExtensionNotLoaded',...
        'StratiGraph extension ''MCS Toolbox Integrator'' is not loaded.')
end

import se.umu.stratigraph.Main;
import se.umu.stratigraph.plugin.extension.mcs.MCSRequestInfoContainer;
import se.umu.stratigraph.plugin.extension.mcs.MCSRequestDataType;
import se.umu.stratigraph.plugin.extension.mcs.MCSRequest;
  
try 
  % Call se.umu.stratigraph.plugin.extension.mcs.MCSRequestDataContainer
  jrc  = MCSRequestInfoContainer(MCSRequestDataType.GET_EXPANDERS, ...
                                 setupName);
  % Call se.umu.stratigraph.plugin.extension.mcs.MCSRequest
  jre  = MCSRequest(winID, jrc);
  
  jres = Main.callExtension(jre); % Get Result
  jstringCont = jres.getData(); % Get StringContainer
  if ~jstringCont.hasData()
    constr = {};
  else
    constr = cell(jstringCont.getStrings());
    constr = cellfun(@(x)strrep(x,'Expander',''),constr,'UniformOutput',false);
  end
catch e
  if isa(e,'matlab.exception.JavaException')
    ex = e.ExceptionObject; % Java exception
    if isa(ex,'java.lang.IllegalStateException')
      error('MCS:sggetconstraints:GetData','StratiGraph returned the error:\n  %s', char(ex.getMessage()))
    else
      rethrow(e)
    end
  else
    rethrow(e)
  end
end

