function varargout = sgopen(label,args)
%SGOPEN Open a StratiGraph window.
%   SGOPEN opens a new StratiGraph window.
%   Label = SGOPEN also returns the label of the StratiGraph window.
%
%   SGOPEN(Label) opens a new StratiGraph window with the label Label that 
%   can be used in communication. The label must either be a positive 
%   integer or a string. If the window Label already exists then it is 
%   activated and send to front.
%
%   SGOPEN(Label,Args) start StratiGraph with the command line arguments
%   Args. Can only be used when no StratiGraph window is open. 
%   For example:
%     sgopen('','-v') - returns the StratiGraph version.
%     sgopen('','-h') - display the help text.
%
%   See also SGSET, SGGET, SGCLOSE.

% Minimum version of StratiGraph supported 
sgMinVerMajor = 4;
sgMinVerMinor = 0;

% Minimum version of Matlab supported 
matlabMinVer = '8.2';
matlabMinRelease = 'R2013b';

narginchk(0,2);
nargoutchk(0,1);

% Check matlab version
if verLessThan('matlab',matlabMinVer)
  error('MCS:sgopen:Version', ['StratiGraph requires at least version ' ...
    matlabMinVer ' (' matlabMinRelease ') of Matlab.'])
end

% Check support for Java
error(javachk('jvm','StratiGraph'))
error(javachk('swing','StratiGraph'))

if nargin < 2 || ~contains(args,{'-v','-h'})
  fprintf(2,'Starting/activating StratiGraph. Please wait...\n');
end

if nargin < 2
  argv = {'-e' 'Matlab Server Communicator' '-e' 'MCS Toolbox Integrator'};
else
  if (ischar(args))
    argv = ['-e' 'Matlab Server Communicator' '-e' 'MCS Toolbox Integrator'...
            strsplit(args,' ')]; 
  else 
    error('MCS:sgopen', 'Args must be a string.') 
  end
end

% Retrieve a list of existing StratiGraph windows
% Also check Java class path and Java version
openedLabels = sglabels();

global MCS_PREF

if ~isfield(MCS_PREF,'sgwin')
  MCS_PREF.sgwin = struct([]);
end

if nargin > 0
  if isnumeric(label)
    if isscalar(label) && label > 0
      label = sprintf('%i',label);
    elseif label == 0
      if isempty(MCS_PREF.sgwin)
        label = '1';
      else
        label = MCS_PREF.sgwin(1).label;
      end
    else
      error('MCS:sgopen', ['The ''label'' argument must be a character, ' ...
             'string, or a positive integer.']);
    end
  elseif ~ischar(label)
    error('MCS:sgopen', ['The ''label'' argument must be a character, ' ...
           'string, or a positive integer.']);
  end
else
  % Find an unused label
  i = 1;
  label = sprintf('%i',i);
  while ismember(label,openedLabels)
    i = i + 1;
    label = sprintf('%i',i);
  end
end

if ~isempty(openedLabels)
  % If StratiGraph is already started...
  sgFrame = stratigraph.getsgFrame();
  err = stratigraph.getErrorMsg();
  if ~isempty(err)
    error(err);
  end
  if ~strcmpi(label,openedLabels)
    sgApp = se.umu.stratigraph.core.util.Application.getApplication();
    sgFrame = sgApp.openWindow(sgFrame,[],label,[]);
  else
    % A SG window with label already exists. Activate it.
    if isempty(MCS_PREF.sgwin)
      tf = 0;
    else
      tf = strcmpi(label,{MCS_PREF.sgwin.label});
    end
    if ~tf
      error('MCS:sgopen', ['A StratiGraph window with label ''%s'' found, '...
             'but it is opened\n outside of Matlab or the '...
             'global variables have been cleared.\n'...
             'Use another label.'],label)
    end
    se.umu.stratigraph.core.util.Application.getApplication().openWindow(sgFrame,MCS_PREF.sgwin(tf).winID,[],[]);
  end		
else
  % Check version of StratiGraph
  try
    ver = se.umu.stratigraph.core.util.Version.getSGVersion();
  catch
    error('MCS:sgopen:Version', ['Probably an old version of StratiGraph ' ...
                        'is in the search path. At least version ' ...
                        '%d.%d is required.'], sgMinVerMajor, sgMinVerMinor)
  end
  
  if (ver.major == sgMinVerMajor)
    if (ver.minor < sgMinVerMinor)
      error('MCS:sgopen:Version', ...
        'The StratiGraph version is %d.%d. At least version %d.%d is required.', ...
        ver.major, ver.minor, sgMinVerMajor, sgMinVerMinor);
    end
  elseif (ver.major < sgMinVerMajor)
    error('MCS:sgopen:Version', ...
      'The StratiGraph version is %d.%d. At least version %d.%d is required.', ...
          ver.major, ver.minor, sgMinVerMajor, sgMinVerMinor);
  end
  
  % Start StratiGraph and check if it returns any error...
  % Call root class stratigraph
  if isempty(argv)
    stratigraph(label);
  else
    stratigraph(label,argv);
  end
  err = stratigraph.getError();
  if ~isempty(err)
    fprintf('\nAn error occurred while opening StratiGraph.\nDo you want to show the stack trace (y/N)? ');
    a = input('','s');
    if isempty(a) || strcmpi(a,'n')
      error(char(err));
    else
      err.printStackTrace
      error(char(err));
    end
  end
  sgFrame = stratigraph.getsgFrame();
  if (isempty(sgFrame))
    % If sgFrame = null exit...
    % Probably StratiGraph is started with the switch -v or -h
    return
  end
end

if isempty(MCS_PREF.sgwin) || ~ismember(label,{MCS_PREF.sgwin(:).label})
  MCS_PREF.sgwin(end+1).label = label;
  MCS_PREF.sgwin(end).winID = sgFrame.getId();
end

if nargout > 0
  varargout{1} = label;
end

