function sgset(varargin)
%SGSET Send a canonical structure to StratiGraph.
%   SGSET(StructObj,StrataType) set the start node in StratiGraph to
%   the canonical structure represented by the object StructObj. Any
%   existing graph in StratiGraph is deleted. The parameter
%   StrataType specifies the strata and must be either 'orbit' or
%   'bundle'. If StrataType is omitted, 'orbit' is assumed. By
%   default, if there exists a graph in StraiGraph the user will be
%   asked if it should be overwritten.
%
%   SGSET(StructObj,StrataType,Constraint) also specify the setup
%   constraint. By default no constraint is used for the setup. To
%   list all available constraints for a canonical structure object
%   call the method GETSGCONSTRAINTS on the object or the function
%   SGGETCONSTRAINTS. By setting Constraint to 'default' or empty
%   string, no constraint is used.
%
%   SGSET(...,ForceOverwrite) let the user supress the prompt if any
%   existing graph in StratiGraph should be overwitten by setting the
%   boolean ForceOverwrite to true.
%
%   SGSET(...,ForceOverwrite,Label) specifies the label of the
%   StratiGraph window to be initialize. If no label is provided the
%   initialization is send to the first window.
%
%   See also SGGET, SGOPEN, SGGETCONSTRAINTS.

nargoutchk(0,0);
  
global MCS_PREF;
  
stratatype = 'orbit';
expanderName = '';
force = false;
label = '';
	
if ~isfield(MCS_PREF,'sgwin')
  error('MCS:sg:NoInstance',...
    ['No instance of StratiGraph is started. Start StratiGraph' ...
    ' with ''sgopen''.']);
end

try
  if ischar(varargin{1})
    remoteData = true;
    switch varargin{1}
     case 'matrix'
      narginchk(2,5);
      A          = varargin{2};
      if nargin > 2
        stratatype = varargin{3};
        if ~ischar(stratatype) | ~strcmpi(stratatype,{'orbit','bundle'}) %#ok<OR2>
          error('MCS:sgset','Unknown strata keyword.');
        end  
      end
      if nargin > 3
        force = varargin{4};
      end
      if nargin == 5
        label = num2str(varargin{5});
      end
      com = sgcom('matrix',A,stratatype);
     case {'abpair','ab','ctrb'}
      narginchk(3,6);
      A          = varargin{2};
      B          = varargin{3};
      if nargin > 3
        stratatype = varargin{4};
        if ~ischar(stratatype) | ~strcmpi(stratatype,{'orbit','bundle'}) %#ok<OR2>
          error('MCS:sgset','Unknown strata keyword.');
        end
      end
      if nargin > 4
        force = varargin{5};
      end
      if nargin == 6
        label = num2str(varargin{6});
      end
      com = sgcom('abpair',A,B,stratatype);
     case {'acpair','ac','obsv'}
      narginchk(3,6);
      A          = varargin{2};
      C          = varargin{3};
      if nargin > 3
        stratatype = varargin{4};
        if ~ischar(stratatype) | ~strcmpi(stratatype,{'orbit','bundle'}) %#ok<OR2>
          error('MCS:sgset','Unknown strata keyword.');
        end
      end
      if nargin > 4
        force = varargin{5};
      end
      if nargin == 6
        label = num2str(varargin{6});
      end
      com = sgcom('acpair',A,C,stratatype);
     case 'pencil'
      narginchk(3,6);
      A          = varargin{2};
      B          = varargin{3};
      if nargin > 3
        stratatype = varargin{4};
        if ~ischar(stratatype) | ~strcmpi(stratatype,{'orbit','bundle'}) %#ok<OR2>
          error('MCS:sgset','Unknown strata keyword.');
        end
      end
      if nargin > 4
        force = varargin{5};
      end
      if nargin == 6
        label = num2str(varargin{6});
      end
      com = sgcom('pencil',A,B,stratatype);
     otherwise
      error('MCS:sgset','Unknown setup type.');
    end
  elseif isobject(varargin{1})
    narginchk(1,5);
    struct = varargin{1};
    if nargin > 1
      stratatype  = varargin{2};
      if ~ischar(stratatype) | ~strcmpi(stratatype,{'orbit','bundle'}) %#ok<OR2>
        error('MCS:sgset','Unknown strata keyword.');
      end
    end
    expPar = 0;
    if nargin > 2 && ~islogical(varargin{3})
      if isempty(varargin{3}) || strcmpi(varargin{3},'default')
        expanderName = '';
      else
        expanderName = [varargin{3} 'Expander'];
      end
      expPar = 1;
    end
    if nargin > 2 + expPar
      force = varargin{3};
    end
    if nargin == 4 + expPar
      label = num2str(varargin{4});
    end
    
    com = sgcom(struct,stratatype);
    remoteData = false;
  else
    error('MCS:sgset','Illegal first argument.');
  end
catch ME
  rethrow(ME);
end

% Associate the sgcom object with the sg window
[~,winID,label] = sgdata(label,com);

% Check that MCS Toolbox Integrator is loaded into StratiGraph
import se.umu.stratigraph.core.PluginManager;
if ~PluginManager.isLoaded(winID,'MCS Toolbox Integrator')
  error('MCS:sgset:ExtensionNotLoaded',...
        'StratiGraph extension ''MCS Toolbox Integrator'' is not loaded for window ''%s''.',label)
end

import se.umu.stratigraph.Main;
import se.umu.stratigraph.core.GraphManager;
import se.umu.stratigraph.plugin.extension.mcs.MCSRequestNewGraphContainer;
import se.umu.stratigraph.plugin.extension.mcs.MCSRequest;
import se.umu.stratigraph.core.plugin.Result;

[setupName,expanderName,setupSize,blockParameters] = com.tojava(expanderName); % Return Java setup parameters
jsm  = jstratmodel(com); % Return Strata
try
  if ~force 
    graph = GraphManager.local(winID);
    if ~isempty(graph) && ~graph.isEmpty()
      a = input('There exists a graph in the StratiGraph window.\nDo you want to replace it with the new structure (Y/n)?','s');
      if strcmpi(a,'n')
        disp('Operation canceled!')
        return
      end
    end
  end
  
  % Call se.umu.stratigraph.plugin.extension.mcs.MCSRequestNewGraphContainer
  jrc  = MCSRequestNewGraphContainer(setupName, expanderName, jsm, blockParameters, setupSize, remoteData);
  % Call se.umu.stratigraph.plugin.extension.mcs.MCSRequest
  jre  = MCSRequest(winID, jrc);
  
  % Activate and send the StratiGraph window to front
  Main.setVisible(winID);
  
  % Update tolerances in StratiGraph
  %[tol,gap] = mcstolerance;
  %sgsettolerance(label,tol,gap);
  
  % Route the request to the MCS Toolbox extension
  jres = Main.callExtension(jre);
  
  if ~jres.isSuccess()
    jres.getErrorCause().printStackTrace();
    error('MCS:sgset:SetData','Error while setting the structure (see stack trace above):\n %s', ...
      char(jres.getErrorCause().toString()))
  end
  
  %if jres.isCanceled()
  %  fprintf(2,'The operation is canceled by user.');
  %end
catch ME
  error('MCS:sgset:SetData',...
        'No response or StratiGraph returned an error:\n%s',getReport(ME));
end
	